<?php
/**
 * @file Rules integration for the taxonomy module.
 *
 * Module sponsored by Made to Order Software Corp.
 *
 * More modules by Made to Order Software Corp.
 * http://www.m2osw.com/drupal
 *
 * @addtogroup rules
 * @{
 */

/**
 * Implementation of hook_rules_event_info().
 */
function tacle_rules_rules_event_info() {
  return array(
    'tacle_rules_term_insert' => array(
      'label' => t('After saving a new term'),
      'module' => 'Tacle Rules',
      'arguments' => tacle_rules_rules_events_hook_taxonomy_term_arguments(t('created term')),
    ),
    'tacle_rules_term_update' => array(
      'label' => t('After updating a term'),
      'module' => 'Tacle Rules',
      'arguments' => tacle_rules_rules_events_hook_taxonomy_term_arguments(t('updated term'), TRUE),
    ),
  );
}

/**
 * Returns some arguments suitable for using it with a term
 */
function tacle_rules_rules_events_hook_taxonomy_term_arguments($term_label, $update = FALSE) {
  $args = array(
    'term' => array(
      'type' => 'taxonomy_term',
      'label' => $term_label,
    ),
  );
  if ($update) {
    $args += array(
      'term_unchanged' => array(
        'type' => 'taxonomy_term',
        'label' => t('unchanged term'),
        'handler' => 'tacle_rules_rules_events_argument_taxonomy_term_unchanged',
      ),
    );
  }
  return $args + rules_events_global_user_argument();
}

/**
 * Gets the term object, that doesn't contain the modified changes
 */
function tacle_rules_rules_events_argument_taxonomy_term_unchanged($taxonomy_term) {
  return $taxonomy_term->tid ? taxonomy_get_term($taxonomy_term->tid) : $taxonomy_term;
}

/**
 * Implementation of hook_rules_action_info().
 */
function tacle_rules_rules_action_info() {
  $info = array();
  // Term actions.
  $info['rules_action_change_role_tacle_rules'] = array(
    'label' => t('Change a role tac_lite access'),
    'arguments' => array(
      //'role' => array(
      //  'type' => 'role',
      //  'label' => t('Role'),
      //),
      'taxonomy_vocab' => array(
        'type' => 'taxonomy_vocab',
        'label' => t('Taxonomy vocabulary'),
      ),
      'taxonomy_term' => array(
        'type' => 'taxonomy_term',
        'label' => t('Taxonomy term'),
      ),
    ),
    'help' => t('Set or remove a permission to a role via Taxonomy Access Lite.'),
    'module' => 'Tacle Rules',
  );
  $info['rules_action_change_user_tacle_rules'] = array(
    'label' => t('Change a user tac_lite access'),
    'arguments' => array(
      'account' => array(
        'type' => 'user',
        'label' => t('User'),
      ),
      'taxonomy_vocab' => array(
        'type' => 'taxonomy_vocab',
        'label' => t('Taxonomy vocabulary'),
      ),
      'taxonomy_term' => array(
        'type' => 'taxonomy_term',
        'label' => t('Taxonomy term'),
      ),
    ),
    'help' => t('Set or remove a permission to a user via Taxonomy Access Lite.'),
    'module' => 'Tacle Rules',
  );
  return $info;
}

/**
 * Action: Setup a tac_lite access to a role.
 */
function rules_action_change_role_tacle_rules($taxonomy_vocab, $taxonomy_term, $settings) {
  $vids = variable_get('tac_lite_categories', null);
  if (!in_array($taxonomy_vocab->vid, $vids)) {
    // tac_lite not setup with that vocabulary
    rules_log(t('The tac_lite module is not handling vocabulary !vid. Please, either fix your rules or the tac_lite setup.', array('!vid' => $taxonomy_vocab->vid)));
    return;
  }

  $roles = $settings['tacle_rules']['roles'];
  $schemes = $settings['tacle_rules']['schemes'];
  if (!count($roles) || !count($schemes)) {
    // no selection, nothing to be done...
    rules_log(t('No roles and/or no schemes were selected for this action. Skipping.'));
    return;
  }

  $max = variable_get('tac_lite_schemes', 1);
  foreach ($schemes as $i) {
    if ($i == 0) {
      // not selected
      continue;
    }
    if ($i > $max) {
      // The scheme defined by the user in the action is invalid
      rules_log(t('The tac_lite scheme #!sid is not available. Skipping.', array('!sid' => $i)));
      continue;
    }

    // Load the roles configuration
    $grants = variable_get('tac_lite_grants_scheme_' . $i, array());

    // The arrays are organized by role, vocabularies, terms.
    // The terms have the ID and value set to the term identifier.
    if ($settings['tacle_rules']['add_permission']) {
      // add the permission
      foreach ($roles as $role) {
        if ($role == 0) {
          // not selected
          continue;
        }
        $grants[$role][$taxonomy_vocab->vid][$taxonomy_term->tid] = $taxonomy_term->tid;
        // remove the <none> selection if present
        unset($grants[$role][$taxonomy_vocab->vid][0]);
      }
    }
    else {
      // remove the permission if present
      foreach ($roles as $role) {
        if ($role == 0) {
          // not selected
          continue;
        }
        unset($grants[$role][$taxonomy_vocab->vid][$taxonomy_term->tid]);
        if (empty($grants[$role][$taxonomy_vocab->vid])) {
          // mark it as "None" instead
          $grants[$role][$taxonomy_vocab->vid][0] = 0;
        }
      }
    }

    variable_set('tac_lite_grants_scheme_' . $i, $grants);
  }
}

/**
 * Action: Setup a tac_lite access to a user.
 */
function rules_action_change_user_tacle_rules(&$account, $taxonomy_vocab, $taxonomy_term, $settings) {
  $vids = variable_get('tac_lite_categories', null);
  if (!in_array($taxonomy_vocab->vid, $vids)) {
    // tac_lite not setup with that vocabulary
    rules_log(t('The tac_lite module is not handling vocabulary !vid. Please, either fix your rules or the tac_lite setup.', array('!vid' => $taxonomy_vocab->vid)));
    return;
  }

  $schemes = $settings['tacle_rules']['schemes'];
  if (!count($schemes)) {
    // no schemes to work on!?
    rules_log(t('No schemes were selected for this action. Skipping.'));
    return;
  }

  $realms = array();
  $max = variable_get('tac_lite_schemes', 1);
  foreach ($schemes as $i) {
    if ($i == 0) {
      // not selected
      continue;
    }
    if ($i > $max) {
      // The scheme defined by the user in the action is invalid
      rules_log(t('The tac_lite scheme #!sid is not available.', array('!sid' => $i)));
      continue;
    }

    // Load the configuration
    $config = _tac_lite_config($i);
    if (!$config['name']) {
      // settings that the user has not yet edited have no name
      rules_log(t('The tac_lite scheme #!sid was not setup yet. Schemes without a name are considered invalid.', array('!sid' => $i)));
      continue;
    }
    $r = $config['realm'];

    // The tac_lite permissions are saved in arrays in the data field of the user account
    $terms = $account->$r;
    if ($settings['tacle_rules']['add_permission']) {
      // add the permission
      if (!$terms) {
        $terms = array();
      }
      if (!in_array($taxonomy_term->tid, $terms[$taxonomy_vocab->vid])) {
        $terms[$taxonomy_vocab->vid][$taxonomy_term->tid] = $taxonomy_term->tid;
        // remove the <none> selection if present
        unset($terms[$taxonomy_vocab->vid][0]);
        $realms[$r] = $terms;
      }
    }
    else {
      // remove the permission if present
      if ($terms && in_array($taxonomy_term->tid, $terms[$taxonomy_vocab->vid])) {
        $terms[$taxonomy_vocab->vid] = array_diff($terms[$taxonomy_vocab->vid], array($taxonomy_term->tid => $taxonomy_term->tid));
        if (!count($terms[$taxonomy_vocab->vid])) {
          // set the <none> entry
          $terms[$taxonomy_vocab->vid][0] = 0;
        }
        $realms[$r] = $terms;
      }
    }
  }

  // save the result now
  user_save($account, $realms);
  $account->data = $realms; // for new users we must save in $account!
}

/**
 * @}
 */
