<?php
/**
 * @file Rules integration for the tac_lite module.
 *
 * Module sponsored by Made to Order Software Corp.
 *
 * More modules by Made to Order Software Corp.
 * http://www.m2osw.com/drupal
 *
 * This file includes the form necessary to setup
 * a tacle entry.
 *
 * @addtogroup rules
 * @{
 */

/**
 * Implementation of hook_rules_action_info().
 */
function rules_action_change_role_tacle_rules_form($settings, &$form, $form_state) {
  $form['settings']['tacle_rules'] = array(
    '#type' => 'fieldset',
    '#title' => t('Taxonomy Access Lite Setup'),
    '#collapsible' => TRUE,
  );

  // Add (checked) or remove (unchecked) permission
  $form['settings']['tacle_rules']['add_permission'] = array(
    '#type' => 'checkbox',
    '#title' => t('Add permission'),
    '#default_value' => !empty($settings['tacle_rules']['add_permission']) ? $settings['tacle_rules']['add_permission'] : 1,
    '#description' => t('Check to add the permission. Uncheck to remove the permission.'),
    '#required' => TRUE,
  );

  // Select which scheme is to be modified
  $form['settings']['tacle_rules']['schemes'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Schemes'),
    '#default_value' => !empty($settings['tacle_rules']['schemes']) ? $settings['tacle_rules']['schemes'] : '',
    '#description' => t('The scheme where the change will be applied.'),
    '#options' => _tacle_rules_get_schemes(),
    '#required' => TRUE,
  );

  // Select the role to be changed
  $form['settings']['tacle_rules']['roles'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Roles'),
    '#options' => user_roles($roles_only),
    '#default_value' => isset($settings['tacle_rules']['roles']) ? $settings['tacle_rules']['roles'] : array(),
    '#required' => TRUE,
  );
}

/**
 * Implementation of hook_rules_action_info().
 */
function rules_action_change_user_tacle_rules_form($settings, &$form, $form_state) {
  $form['settings']['tacle_rules'] = array(
    '#type' => 'fieldset',
    '#title' => t('Taxonomy Access Lite Setup'),
    '#collapsible' => TRUE,
  );

  // Add (checked) or remove (unchecked) permission
  $form['settings']['tacle_rules']['add_permission'] = array(
    '#type' => 'checkbox',
    '#title' => t('Add permission'),
    '#default_value' => !empty($settings['tacle_rules']['add_permission']) ? $settings['tacle_rules']['add_permission'] : 1,
    '#description' => t('Check to add the permission. Uncheck to remove the permission'),
    '#required' => TRUE,
  );

  // Select which scheme is to be modified
  $form['settings']['tacle_rules']['schemes'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Schemes'),
    '#default_value' => !empty($settings['tacle_rules']['schemes']) ? $settings['tacle_rules']['schemes'] : '',
    '#description' => t('The scheme where the change will be applied.'),
    '#options' => _tacle_rules_get_schemes(),
    '#required' => TRUE,
  );
}

/**
 * Helper function used to generate the array of schemes for the forms.
 */
function _tacle_rules_get_schemes() {
  $schemes = array();

  $max = variable_get('tac_lite_schemes', 1);
  for ($i = 1; $i <= $max; ++$i) {
    $schemes[$i] = t('Scheme') . ' ' . $i . ': ';

    $config = _tac_lite_config($i);
    if ($config['name']) {
      $schemes[$i] .= $config['name'];
    }
    else {
      $schemes[$i] .= t('not available');
    }
  }

  return $schemes;
}

/**
 * @}
 */

